package net.wanji.utc.service.staticinfo.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.CrossSchedulesDTO;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.vo.CrossIdVO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.Result;
import net.wanji.utc.service.staticinfo.WanJiCommonStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/09 9:21
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class WanJiCommonStaticInfoServiceImpl implements WanJiCommonStaticInfoService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossInfoCache crossInfoCache;
    private final UtcFeignClientCache utcFeignClientCache;


    @Override
    public void schemePhaseLights(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossIdVO crossIdVO = new CrossIdVO();
        crossIdVO.setCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).schemePhaseLights(crossIdVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取方案数据-方案信息、相位信息、灯组信息远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper objectMapper = new ObjectMapper();
        SchemePhaseLightsVO content = objectMapper.convertValue(jsonViewObject.getContent(), SchemePhaseLightsVO.class);
        List<BaseCrossSchemePO> crossSchemeList = content.getCrossSchemeList();
        crossSchemeList.forEach(crossSchemePO -> {
            crossSchemeMapper.deleteOne(crossId, crossSchemePO.getSchemeNo());
            crossSchemeMapper.insertOne(crossSchemePO);
        });
        List<CrossPhasePO> crossPhaseList = content.getCrossPhaseList();
        crossPhaseMapper.deleteBatch(crossId, null);
        crossPhaseList.forEach(crossPhasePO -> {
            BaseCrossSchemePO baseCrossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, crossPhasePO.getPlanId());
            Integer id;
            if (Objects.isNull(baseCrossSchemePO)) {
                id = -1;
            } else {
                id = baseCrossSchemePO.getId();
            }
            crossPhasePO.setPlanId(id);
            crossPhaseMapper.insertOne(crossPhasePO);
        });

        List<CrossLightsPO> crossLightsList = content.getCrossLightsList();
        if (!CollectionUtils.isEmpty(crossLightsList)) {
            crossLightsMapper.deleteByCrossId(crossId);
            crossLightsList.forEach(crossLightsPO -> {
                crossLightsMapper.insertOne(crossLightsPO);
            });
        }
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = content.getCrossPhaseLightsPOList();
        if (!CollectionUtils.isEmpty(crossPhaseLightsPOList)) {
            crossPhaseLightsMapper.deleteByCrossId(crossPhaseLightsPOList.get(0).getCrossId());
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                String currentCrossId = crossPhaseLightsPO.getCrossId();
                Integer lightsId = crossPhaseLightsPO.getLightsId();
                Integer phaseId = crossPhaseLightsPO.getPhaseId();
                List<Integer> list = crossPhaseMapper.selectIdsByPhaseNo(currentCrossId, phaseId);
                if (!CollectionUtils.isEmpty(list)) {
                    crossPhaseLightsPO.setPhaseId(list.get(0));
                }
                CrossLightsPO crossLightsPO = new CrossLightsPO();
                crossLightsPO.setCrossId(currentCrossId);
                crossLightsPO.setLightsNo(String.valueOf(lightsId));
                CrossLightsPO result = crossLightsMapper.selectEntity(crossLightsPO);
                crossPhaseLightsPO.setLightsId(result.getId());
            }
            crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
        }
    }

    @Override
    public void planSection(String crossId, String signalId, Integer planNo) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        PlanSectionDTO planSectionDTO = new PlanSectionDTO();
        planSectionDTO.setCrossId(crossId);
        planSectionDTO.setPlanNo(planNo);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).planSection(planSectionDTO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取计划数据-计划信息、时段信息远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper mapper = new ObjectMapper();
        List<PlanSectionVO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<PlanSectionVO>>() {});
        if (!CollectionUtils.isEmpty(content)) {
            for (PlanSectionVO planSectionVO : content) {
                List<CrossPlanPO> crossPlanPOList = planSectionVO.getCrossPlanPOList();
                crossPlanPOList.forEach(crossPlanPO -> {
                    crossPlanMapper.deleteOne(crossId, Integer.parseInt(crossPlanPO.getPlanNo()));
                    crossPlanMapper.insertOne(crossPlanPO);
                });

                List<CrossSectionPO> crossSectionPOList = planSectionVO.getCrossSectionPOList();
                crossSectionPOList.forEach(crossSectionPO -> {
                    crossSectionMapper.deleteOne(crossId, Integer.parseInt(crossSectionPO.getSectionNo()), null);
                    Integer id = crossPlanMapper.selectIdByNo(crossSectionPO.getCrossId(), crossSectionPO.getPlanId());
                    crossSectionPO.setPlanId(id);
                    BaseCrossSchemePO baseCrossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, crossSectionPO.getSchemeId());
                    crossSectionPO.setSchemeId(baseCrossSchemePO.getId());
                    crossSectionMapper.insertOne(crossSectionPO);
                });
            }
        }
    }

    @Override
    public Result<List<CrossSchedulesPO>> schedules(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossSchedulesDTO crossSchedulesDTO = new CrossSchedulesDTO();
        crossSchedulesDTO.setManufacturerCode(manufacturerIdCode);
        crossSchedulesDTO.setCrossIdList(Arrays.asList(crossId));
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).crossSchedules(crossId);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取时间表数据远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper mapper = new ObjectMapper();
        List<CrossSchedulesPO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<CrossSchedulesPO>>() {});

        if (Objects.nonNull(content)) {
            crossSchedulesMapper.deleteByCrossId(crossId);
            if (!CollectionUtils.isEmpty(content)) {
                for (CrossSchedulesPO crossSchedulesPO : content) {
                    Integer planId = crossSchedulesPO.getPlanId();
                    Integer id = crossPlanMapper.selectIdByNo(crossId, planId);
                    crossSchedulesPO.setPlanId(id);
                }
                crossSchedulesMapper.insertBatch(content);
            }
        }
        return Result.success(content);
    }

    public static Map<String, String> getCommonParamMap(String crossId, String signalId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("crossId", crossId);
        paramMap.put("signalId", signalId);
        return paramMap;
    }

}
